@@ -37,6 +37,7 @@
 #include <math.h>
 #include <time.h>
 #include <unistd.h>
+#include <ctype.h>
 
 #include <gdk/gdkkeysyms.h>
 #include <gtk/gtk.h>
@@ -300,6 +301,8 @@ void handle_buddy_rename(struct buddy *b, char *prevname)
 		}
 		update_num_group(gs);
 	} else {
+		if (misc_options & OPT_MISC_BUDDY_TICKER)
+			BuddyTickerSetAlias(b->name, b->show);
 		gtk_label_set_text(GTK_LABEL(bs->label), b->show);
 		update_idle_time(bs);
 	}
@@ -610,12 +613,9 @@ void pressed_alias_bud(GtkWidget *widget, struct buddy *b)
 static void menu_click(GtkObject *obj, char *who)
 {
 	GList *list = gtk_object_get_user_data(obj);
-	GList *first = g_list_first(list);
 	struct proto_buddy_menu *pbm = list->data;
 	if (pbm->callback)
 		pbm->callback(pbm->gc, who);
-	g_list_foreach(first, (GFunc)g_free, NULL);
-	g_list_free(first);
 }
 
 static int handle_click_buddy(GtkWidget *widget, GdkEventButton *event, struct buddy_show *b)
@@ -636,7 +636,8 @@ static int handle_click_buddy(GtkWidget *widget, GdkEventButton *event, struct b
 		if (im_options & OPT_IM_ONE_WINDOW)
 			raise_convo_tab(c);
 	} else if (event->type == GDK_BUTTON_PRESS && event->button == 3) {
-		GtkWidget *menu;
+		static GtkWidget *menu = NULL;
+		static GList *mo_top = NULL;
 		GtkWidget *button;
 		GtkWidget *menuitem;
 		GtkWidget *conmenu;
@@ -644,6 +645,20 @@ static int handle_click_buddy(GtkWidget *widget, GdkEventButton *event, struct b
 		struct gaim_connection *g;
 		/* We're gonna make us a menu right here */
 
+		/*
+		 * If a menu already exists, destroy it before creating a new one,
+		 * thus freeing-up the memory it occupied.  Same for its associated
+		 * (prpl menu items) GList.
+		 */
+		if(menu) {
+			gtk_widget_destroy(menu);
+			if(mo_top) {
+				g_list_foreach(mo_top, (GFunc)g_free, NULL);
+				g_list_free(mo_top);
+				mo_top = NULL;
+			}
+		}
+
 		menu = gtk_menu_new();
 
 		button = gtk_menu_item_new_with_label(_("IM"));
@@ -673,7 +688,7 @@ static int handle_click_buddy(GtkWidget *widget, GdkEventButton *event, struct b
 			while (cn) {
 				g = (struct gaim_connection *)cn->data;
 				if (g->prpl->buddy_menu) {
-					GList *mo = g->prpl->buddy_menu(g, b->name);
+					GList *mo = mo_top = g->prpl->buddy_menu(g, b->name);
 
 					menuitem = gtk_menu_item_new_with_label(g->username);
 					gtk_menu_append(GTK_MENU(menu), menuitem);
@@ -702,7 +717,7 @@ static int handle_click_buddy(GtkWidget *widget, GdkEventButton *event, struct b
 		} else {
 			g = (struct gaim_connection *)cn->data;
 			if (g->prpl->buddy_menu) {
-				GList *mo = g->prpl->buddy_menu(g, b->name);
+				GList *mo = mo_top = g->prpl->buddy_menu(g, b->name);
 
 				while (mo) {
 					struct proto_buddy_menu *pbm = mo->data;
@@ -772,8 +787,9 @@ static gboolean click_edit_tree(GtkWidget *widget, GdkEventButton *event, gpoint
 	GtkCTreeNode *node;
 	int *type;
 	int row, column;
-	GtkWidget *menu;
+	static GtkWidget *menu = NULL;
 	GtkWidget *button;
+	static GList *mo_top = NULL;
 
 	if (event->button != 3 || event->type != GDK_BUTTON_PRESS)
 		return FALSE;
@@ -783,6 +799,21 @@ static gboolean click_edit_tree(GtkWidget *widget, GdkEventButton *event, gpoint
 
 	node = gtk_ctree_node_nth(GTK_CTREE(edittree), row);
 	type = gtk_ctree_node_get_row_data(GTK_CTREE(edittree), node);
+
+	/*
+	 * If a menu already exists, destroy it before creating a new one,
+	 * thus freeing-up the memory it occupied.
+	 */
+	if(menu) {
+		gtk_widget_destroy(menu);
+		menu = NULL;	/* safety measure */
+		if(mo_top) {
+			g_list_foreach(mo_top, (GFunc)g_free, NULL);
+			g_list_free(mo_top);
+			mo_top = NULL;
+		}
+	}
+
 	if (*type == EDIT_GROUP) {
 		struct group *group = (struct group *)type;
 		menu = gtk_menu_new();
@@ -836,6 +867,27 @@ static gboolean click_edit_tree(GtkWidget *widget, GdkEventButton *event, gpoint
 		gtk_menu_append(GTK_MENU(menu), button);
 		gtk_widget_show(button);
 
+		/*
+		 * Add protocol-specific edit buddy menu items if they exist
+		 */
+		if (b->gc->prpl->edit_buddy_menu) {
+			GList *mo = mo_top = b->gc->prpl->edit_buddy_menu(b->gc, b->name);
+
+			while (mo) {
+				struct proto_buddy_menu *pbm = mo->data;
+				GtkWidget *button;
+
+				button = gtk_menu_item_new_with_label(pbm->label);
+				gtk_signal_connect(GTK_OBJECT(button), "activate",
+						   GTK_SIGNAL_FUNC(menu_click), b->name);
+				gtk_object_set_user_data(GTK_OBJECT(button), mo);
+				gtk_menu_append(GTK_MENU(menu), button);
+				gtk_widget_show(button);
+
+				mo = mo->next;
+			}
+		}
+
 		gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, event->button, event->time);
 
 		return TRUE;
@@ -1507,12 +1559,19 @@ void chat_callback(GtkWidget *widget, GtkTree *tree)
 static void away_callback(GtkWidget *widget, GtkTree *tree)
 {
 	GSList *awy = away_messages;
-	GtkWidget *menu;
+	static GtkWidget *menu = NULL;
 	GtkWidget *menuitem;
 
 	if (!awy)
 		return;
 
+	/*
+	 * If a menu already exists, destroy it before creating a new one,
+	 * thus freeing-up the memory it occupied.
+	 */
+	if(menu)
+		gtk_widget_destroy(menu);
+
 	menu = gtk_menu_new();
 
 	while (awy) {
@@ -1523,11 +1582,10 @@ static void away_callback(GtkWidget *widget, GtkTree *tree)
 		gtk_signal_connect(GTK_OBJECT(menuitem), "activate",
 				   GTK_SIGNAL_FUNC(do_away_message), a);
 		gtk_widget_show(menuitem);
-
 		awy = awy->next;
 	}
 
-	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 1, time(NULL));
+	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, NULL, NULL, 1, GDK_CURRENT_TIME );
 }
 
 void rem_bp(GtkWidget *w, struct buddy_pounce *b)
@@ -1579,9 +1637,9 @@ void do_pounce(struct gaim_connection *gc, char *name, int when)
 				 * because I thought it'd be funny :-) */
 
 				g_snprintf(tmp, sizeof(tmp), "%s has %s", name, 
-					   (b->options & OPT_POUNCE_TYPING) ? "started typing to you" :
-					   (b->options & OPT_POUNCE_SIGNON) ? "signed on" : 
-					   (b->options & OPT_POUNCE_UNIDLE) ? "returned from being idle" : 
+					   (when & OPT_POUNCE_TYPING) ? "started typing to you" :
+					   (when & OPT_POUNCE_SIGNON) ? "signed on" : 
+					   (when & OPT_POUNCE_UNIDLE) ? "returned from being idle" : 
 					   "returned from being away");
 				
 				do_error_dialog(tmp, _("Buddy Pounce"));
@@ -1975,23 +2033,28 @@ static struct group_show *find_gs_by_bs(struct buddy_show *b)
 }
 
 void hide_buddy_list() {
+	if(blist) {
 #ifdef USE_APPLET
-	applet_destroy_buddy(NULL, NULL, NULL);
+		applet_destroy_buddy(NULL, NULL, NULL);
 #else
-	XIconifyWindow(GDK_DISPLAY(),
+		XIconifyWindow(GDK_DISPLAY(),
 			GDK_WINDOW_XWINDOW(blist->window),
 			((_XPrivDisplay)GDK_DISPLAY())->default_screen);
 #endif
+	}
 }
 
 void unhide_buddy_list() {
+	if(blist) {
 #ifdef USE_APPLET
-	if (!applet_buddy_show) {
-		applet_buddy_show = TRUE;
-		createOnlinePopup();
-	}
+		if (!applet_buddy_show) {
+			applet_buddy_show = TRUE;
+			createOnlinePopup();
+		}
 #endif /* USE_APPLET */
-	gdk_window_show(blist->window);
+		if(blist)
+			gdk_window_show(blist->window);
+	}
 }
 
 static gint log_timeout(struct buddy_show *b)
@@ -2285,7 +2348,7 @@ void set_buddy(struct gaim_connection *gc, struct buddy *b)
 			gtk_pixmap_set(GTK_PIXMAP(bs->pix), pm, bm);
 			gtk_widget_show(bs->pix);
 			if (misc_options & OPT_MISC_BUDDY_TICKER) {
-				BuddyTickerAddUser(b->name, pm, bm);
+				BuddyTickerAddUser(b->name, b->show, pm, bm);
 				gtk_timeout_add(10000, (GtkFunction)BuddyTickerLogonTimeout, b->name);
 			}
 			gdk_pixmap_unref(pm);
