@@ -714,17 +714,21 @@ void chat_write(struct conversation *b, char *who, int flag, char *message, time
 				play_sound(SND_CHAT_YOU_SAY);
 			flag |= WFLAG_SEND;
 		} else {
-			if (b->makesound)
-				play_sound(SND_CHAT_SAY);
-			flag |= WFLAG_RECV;
+		       	flag |= WFLAG_RECV;
+			if (find_nick(b->gc, message))
+				flag |= WFLAG_NICK;
 		}
 		g_free(str);
 	}
 
-	if ((flag & WFLAG_RECV) && find_nick(b->gc, message))
-		flag |= WFLAG_NICK;
-
-	write_to_conv(b, message, flag, who, mtime, -1);
+	if (flag & WFLAG_RECV && b->makesound) {
+		if (flag & WFLAG_NICK && (sound_options & OPT_SOUND_CHAT_NICK)) {
+			play_sound(SND_CHAT_NICK);
+		} else {
+			play_sound(SND_CHAT_SAY);
+		}
+	}
+		write_to_conv(b, message, flag, who, mtime, -1);
 }
 
 
@@ -772,17 +776,17 @@ static gint insertname(gconstpointer one, gconstpointer two)
 	if (*a == '@') {
 		if (*b != '@')
 			return -1;
-		return (strcmp(a + 1, b + 1));
+		return (strcasecmp(a + 1, b + 1));
 	} else if (*a == '+') {
 		if (*b == '@')
 			return 1;
 		if (*b != '+')
 			return -1;
-		return (strcmp(a + 1, b + 1));
+		return (strcasecmp(a + 1, b + 1));
 	} else {
 		if (*b == '@' || *b == '+')
 			return 1;
-		return strcmp(a, b);
+		return strcasecmp(a, b);
 	}
 }
 
@@ -842,9 +846,16 @@ static gint right_click_chat(GtkObject *obj, GdkEventButton *event, struct conve
 			c = new_conversation(gtk_object_get_user_data(obj));
 		set_convo_gc(c, b->gc);
 	} else if (event->button == 3 && event->type == GDK_BUTTON_PRESS) {
-		GtkWidget *menu;
+		static GtkWidget *menu = NULL;
 		GtkWidget *button;
 
+		/*
+		 * If a menu already exists, destroy it before creating a new one,
+		 * thus freeing-up the memory it occupied.
+		 */
+		if(menu)
+			gtk_widget_destroy(menu);
+
 		menu = gtk_menu_new();
 
 		button = gtk_menu_item_new_with_label(_("IM"));
