@@ -28,6 +28,9 @@
 
 #include "core.h"
 #include "ui.h"
+
+#define XPATCH BAD /* Because Kalla Said So */
+
 /* XXX CUI: when this is done being split, the only things below should be things
  * both the core and the uis depend on e.g. the protocol definitions, etc, and
  * it won't include core.h or ui.h (i.e. it'll mostly be #define's) */
@@ -208,6 +211,7 @@ extern guint blist_options;
 #define OPT_BLIST_SHOW_WARN		0x00000200
 #define OPT_BLIST_GREY_IDLERS		0x00000400
 #define OPT_BLIST_BOTTOM_TAB		0X00000800
+#define OPT_BLIST_POPUP                 0x00001000
 
 extern guint convo_options;
 #define OPT_CONVO_ENTER_SENDS		0x00000001
@@ -224,6 +228,7 @@ extern guint convo_options;
 #define OPT_CONVO_IGNORE_FONTS		0x00000800
 #define OPT_CONVO_IGNORE_SIZES		0x00001000
 #define OPT_CONVO_COMBINE		0x00002000
+#define OPT_CONVO_CTL_W_CLOSES          0x00004000
 
 extern guint im_options;
 #define OPT_IM_POPUP			0x00000001
@@ -278,7 +283,7 @@ extern guint sound_options;
 #define OPT_SOUND_ARTSC			0x00008000
 #define OPT_SOUND_ESD			0x00010000
 #define OPT_SOUND_CMD			0x00020000
-
+#define OPT_SOUND_CHAT_NICK             0x00040000
 /* remember to also change the struct in sound.c */
 #define SND_BUDDY_ARRIVE 0
 #define SND_BUDDY_LEAVE 1
@@ -290,7 +295,11 @@ extern guint sound_options;
 #define SND_CHAT_YOU_SAY 7
 #define SND_CHAT_SAY 8
 #define SND_POUNCE_DEFAULT 9
-#define NUM_SOUNDS 10
+#define SND_CHAT_NICK 10
+#define NUM_SOUNDS 11
+/* these two for the sound_order list in prefs.c */
+#define SND_SEPARATOR -1
+#define SND_END -2
 extern char *sound_file[NUM_SOUNDS];
 
 /* global sound struct */
@@ -339,6 +348,8 @@ extern void remove_chat_buddy(struct conversation *, char *, char *);
 /* Functions in conversation.c */
 extern void write_to_conv(struct conversation *, char *, int, char *, time_t, int);
 extern struct conversation *find_conversation(char *);
+extern void set_anim();
+extern void set_blist_tab();
 
 /* Functions in dialogs.c */
 extern void g_show_info_text(struct gaim_connection *, char *, int, char *, ...);
@@ -396,6 +407,8 @@ extern void serv_chat_whisper(struct gaim_connection *, int, char *, char *);
 extern int  serv_chat_send(struct gaim_connection *, int, char *);
 extern void serv_got_popup(char *, char *, int, int);
 extern void serv_get_away(struct gaim_connection *, char *);
+extern void serv_alias_buddy(struct buddy *);
+extern void serv_move_buddy(struct buddy *, struct group *, struct group *);
 
 /* Functions in util.c */
 extern char *normalize(const char *);
@@ -424,6 +437,15 @@ extern char *str_to_utf8(unsigned char *);
 extern char *add_cr(char *);
 extern void strip_linefeed(char *);
 extern time_t get_time(int, int, int, int, int, int);
+extern FILE *gaim_mkstemp(gchar **);
+
+#ifdef HAVE_LANGINFO_CODESET
+#define utf8_to_str(in) convert_string(in, nl_langinfo(CODESET), "UTF-8");
+#define str_to_utf8(in) convert_string(in, "UTF-8", nl_langinfo(CODESET));
+#else
+#define utf8_to_str(in) convert_string(in, "ISO-8859-1", "UTF-8");
+#define str_to_utf8(in) convert_string(in, "UTF-8", "ISO-8859-1");
+#endif
 
 /*------------------------------------------------------------------------*/
 /*  Multi-Entry dialog and vCard dialog support                           */
