@@ -63,16 +63,15 @@ struct parse {
 	char value[MAX_VALUES][4096];
 };
 
-static struct parse *parse_line(char *line)
+static struct parse *parse_line(char *line, struct parse *p)
 {
 	char *c = line;
 	int inopt = 1, inval = 0, curval = -1;
 	int optlen = 0, vallen = 0;
-	static struct parse p;
 	int x;
 
 	for (x = 0; x < MAX_VALUES; x++) {
-		p.value[x][0] = 0;
+		p->value[x][0] = 0;
 	}
 
 
@@ -85,36 +84,36 @@ static struct parse *parse_line(char *line)
 			/*   if ((*c < 'a' || *c > 'z') && *c != '_') { */
 			if ((*c < 'a' || *c > 'z') && *c != '_' && (*c < 'A' || *c > 'Z')) {
 				inopt = 0;
-				p.option[optlen] = 0;
+				p->option[optlen] = 0;
 				c++;
 				continue;
 			}
 
-			p.option[optlen] = *c;
+			p->option[optlen] = *c;
 			optlen++;
 			c++;
 			continue;
 		} else if (inval) {
 			if ((*c == '}')) {
 				if (*(c - 1) == '\\') {
-					p.value[curval][vallen - 1] = *c;
+					p->value[curval][vallen - 1] = *c;
 					c++;
 					continue;
 				} else {
-					p.value[curval][vallen - 1] = 0;
+					p->value[curval][vallen - 1] = 0;
 					inval = 0;
 					c++;
 					continue;
 				}
 			} else {
-				p.value[curval][vallen] = *c;
+				p->value[curval][vallen] = *c;
 				vallen++;
 				c++;
 				continue;
 			}
 		} else if (*c == '{') {
 			if (*(c - 1) == '\\') {
-				p.value[curval][vallen - 1] = *c;
+				p->value[curval][vallen - 1] = *c;
 				c++;
 				continue;
 			} else {
@@ -129,7 +128,7 @@ static struct parse *parse_line(char *line)
 		c++;
 	}
 
-	return &p;
+	return p;
 }
 
 
@@ -228,6 +227,7 @@ static char *escape_text2(const char *msg)
 
 static void gaimrc_read_away(FILE *f)
 {
+	struct parse parse_buffer;
 	struct parse *p;
 	char buf[4096];
 	struct away_message *a;
@@ -241,7 +241,7 @@ static void gaimrc_read_away(FILE *f)
 		if (buf[0] == '}')
 			return;
 
-		p = parse_line(buf);
+		p = parse_line(buf, &parse_buffer);
 		if (!strcmp(p->option, "message")) {
 			a = g_new0(struct away_message, 1);
 
@@ -301,6 +301,7 @@ static void gaimrc_write_away(FILE *f)
 
 static void gaimrc_read_pounce(FILE *f)
 {
+	struct parse parse_buffer;
 	struct parse *p;
 	char buf[4096];
 	struct buddy_pounce *b;
@@ -314,7 +315,7 @@ static void gaimrc_read_pounce(FILE *f)
 		if (buf[0] == '}')
 			return;
 
-		p = parse_line(buf);
+		p = parse_line(buf, &parse_buffer);
 		if (!strcmp(p->option, "entry")) {
 			b = g_new0(struct buddy_pounce, 1);
 
@@ -413,6 +414,7 @@ static void gaimrc_write_plugins(FILE *f)
 
 static void gaimrc_read_plugins(FILE *f)
 {
+	struct parse parse_buffer;
 	struct parse *p;
 	char buf[4096];
 	GSList *load = NULL;
@@ -426,7 +428,7 @@ static void gaimrc_read_plugins(FILE *f)
 		if (buf[0] == '}')
 			break;
 
-		p = parse_line(buf);
+		p = parse_line(buf, &parse_buffer);
 		if (!strcmp(p->option, "plugin")) {
 			filter_break(p->value[0]);
 			load = g_slist_append(load, g_strdup(p->value[0]));
@@ -446,6 +448,7 @@ static void gaimrc_read_plugins(FILE *f)
 
 static struct aim_user *gaimrc_read_user(FILE *f)
 {
+	struct parse parse_buffer;
 	struct parse *p;
 	struct aim_user *u;
 	int i;
@@ -454,7 +457,7 @@ static struct aim_user *gaimrc_read_user(FILE *f)
 	if (!fgets(buf, sizeof(buf), f))
 		return NULL;
 
-	p = parse_line(buf);
+	p = parse_line(buf, &parse_buffer);
 
 	if (strcmp(p->option, "ident"))
 		return NULL;
@@ -499,7 +502,7 @@ static struct aim_user *gaimrc_read_user(FILE *f)
 		return u;
 	}
 
-	p = parse_line(buf);
+	p = parse_line(buf, &parse_buffer);
 
 	if (strcmp(p->option, "user_opts"))
 		return u;
@@ -513,7 +516,7 @@ static struct aim_user *gaimrc_read_user(FILE *f)
 	if (!strcmp(buf, "\t}"))
 		return u;
 
-	p = parse_line(buf);
+	p = parse_line(buf, &parse_buffer);
 
 	if (strcmp(p->option, "proto_opts"))
 		return u;
@@ -527,7 +530,7 @@ static struct aim_user *gaimrc_read_user(FILE *f)
 	if (!strcmp(buf, "\t}"))
 		return u;
 
-	p = parse_line(buf);
+	p = parse_line(buf, &parse_buffer);
 
 	if (strcmp(p->option, "iconfile"))
 		return u;
@@ -540,7 +543,7 @@ static struct aim_user *gaimrc_read_user(FILE *f)
 	if (!strcmp(buf, "\t}"))
 		return u;
 
-	p = parse_line(buf);
+	p = parse_line(buf, &parse_buffer);
 
 	if (strcmp(p->option, "alias"))
 		return u;
@@ -555,10 +558,13 @@ static void gaimrc_write_user(FILE *f, struct aim_user *u)
 {
 	char *c;
 	int nl = 1, i;
-	if (u->options & OPT_USR_REM_PASS)
-		fprintf(f, "\t\tident { %s } { %s }\n", u->username, u->password);
-	else
+
+	if (u->options & OPT_USR_REM_PASS) {
+		fprintf(f, "\t\tident { %s } { %s }\n", u->username, (c = escape_text2(u->password)));
+		free(c);
+	} else {
 		fprintf(f, "\t\tident { %s } {  }\n", u->username);
+	}
 	fprintf(f, "\t\tuser_info {");
 	c = u->user_info;
 	while (*c) {
@@ -590,6 +596,7 @@ static void gaimrc_read_users(FILE *f)
 {
 	char buf[2048];
 	struct aim_user *u;
+	struct parse parse_buffer;
 	struct parse *p;
 
 	buf[0] = 0;
@@ -603,7 +610,7 @@ static void gaimrc_read_users(FILE *f)
 
 
 
-		p = parse_line(buf);
+		p = parse_line(buf, &parse_buffer);
 
 		if (!strcmp(p->option, "current_user")) {
 		} else if (strcmp(p->option, "user")) {
@@ -704,6 +711,7 @@ static struct replace disp_replace[] = {
 static void gaimrc_read_options(FILE *f)
 {
 	char buf[2048];
+	struct parse parse_buffer;
 	struct parse *p;
 	gboolean read_logging = FALSE, read_general = FALSE, read_display = FALSE;
 	int general_options = 0, display_options = 0;
@@ -718,7 +726,7 @@ static void gaimrc_read_options(FILE *f)
 		if (!fgets(buf, sizeof(buf), f))
 			return;
 
-		p = parse_line(buf);
+		p = parse_line(buf, &parse_buffer);
 
 		if (!strcmp(p->option, "general_options")) {
 			general_options = atoi(p->value[0]);
@@ -866,6 +874,7 @@ static void gaimrc_read_sounds(FILE *f)
 {
 	int i;
 	char buf[2048];
+	struct parse parse_buffer;
 	struct parse *p;
 
 	buf[0] = 0;
@@ -881,7 +890,7 @@ static void gaimrc_read_sounds(FILE *f)
 		if (!fgets(buf, sizeof(buf), f))
 			return;
 
-		p = parse_line(buf);
+		p = parse_line(buf, &parse_buffer);
 
 		if (!strcmp(p->option, "sound_cmd")) {
 			g_snprintf(sound_cmd, sizeof(sound_cmd), "%s", p->value[0]);
@@ -907,9 +916,100 @@ static void gaimrc_write_sounds(FILE *f)
 	fprintf(f, "}\n");
 }
 
+static gboolean gaimrc_parse_proxy_uri(const char *proxy)
+{
+	char *c, *d;
+	char buffer[2048];
+
+	char host[128];
+	char user[128];
+	char pass[128];
+	int  port = 0;
+	int  len  = 0;
+
+	if ((c = strchr(proxy, ':')) == NULL)
+	{
+		/* No URI detected. */
+		return FALSE;
+	}
+
+	len = c - proxy;
+
+	if (!strncmp(proxy, "http", len))
+		proxytype = PROXY_HTTP;
+	else
+		return FALSE;
+
+	/* Get past "://" */
+	c += 3;
+
+	for (;;)
+	{
+		*buffer = '\0';
+		d = buffer;
+
+		while (*c != '\0' && *c != '@' && *c != ':' && *c != '/')
+			*d++ = *c++;
+
+		*d = '\0';
+
+		if (*c == ':')
+		{
+			/*
+			 * If there is a '@' in there somewhere, we are in the auth part.
+			 * If not, host.
+			 */
+			if (strchr(c, '@') != NULL)
+				strcmp(user, buffer);
+			else
+				strcmp(host, buffer);
+		}
+		else if (*c == '@')
+		{
+			if (user == NULL)
+				strcmp(user, buffer);
+			else
+				strcmp(pass, buffer);
+		}
+		else if (*c == '/' || *c == '\0')
+		{
+			if (host == NULL)
+				strcmp(host, buffer);
+			else
+				port = atoi(buffer);
+
+			/* Done. */
+			break;
+		}
+
+		c++;
+	}
+
+	/* NOTE: HTTP_PROXY takes precendence. */
+	if (host)
+		strcpy(proxyhost, host);
+	else
+		*proxyhost = '\0';
+
+	if (user)
+		strcpy(proxyuser, user);
+	else
+		*proxyuser = '\0';
+
+	if (pass)
+		strcpy(proxypass, pass);
+	else
+		*proxypass = '\0';
+	
+	proxyport = port;
+
+	return TRUE;
+}
+
 static void gaimrc_read_proxy(FILE *f)
 {
 	char buf[2048];
+	struct parse parse_buffer;
 	struct parse *p;
 
 	buf[0] = 0;
@@ -922,7 +1022,7 @@ static void gaimrc_read_proxy(FILE *f)
 		if (!fgets(buf, sizeof(buf), f))
 			return;
 
-		p = parse_line(buf);
+		p = parse_line(buf, &parse_buffer);
 
 		if (!strcmp(p->option, "host")) {
 			g_snprintf(proxyhost, sizeof(proxyhost), "%s", p->value[0]);
@@ -937,6 +1037,8 @@ static void gaimrc_read_proxy(FILE *f)
 		}
 	}
 	if (!proxyhost[0]) {
+		gboolean getVars = TRUE;
+		
 		if (g_getenv("HTTP_PROXY"))
 			g_snprintf(proxyhost, sizeof(proxyhost), "%s", g_getenv("HTTP_PROXY"));
 		else if (g_getenv("http_proxy"))
@@ -944,40 +1046,50 @@ static void gaimrc_read_proxy(FILE *f)
 		else if (g_getenv("HTTPPROXY"))
 			g_snprintf(proxyhost, sizeof(proxyhost), "%s", g_getenv("HTTPPROXY"));
 
-		if (g_getenv("HTTP_PROXY_PORT"))
-			proxyport = atoi(g_getenv("HTTP_PROXY_PORT"));
-		else if (g_getenv("http_proxy_port"))
-			proxyport = atoi(g_getenv("http_proxy_port"));
-		else if (g_getenv("HTTPPROXYPORT"))
-			proxyport = atoi(g_getenv("HTTPPROXYPORT"));
+		if (*proxyhost != '\0')
+			getVars = !gaimrc_parse_proxy_uri(proxyhost);
 
-		if (g_getenv("HTTP_PROXY_USER"))
-			g_snprintf(proxyuser, sizeof(proxyuser), "%s", g_getenv("HTTP_PROXY_USER"));
-		else if (g_getenv("http_proxy_user"))
-			g_snprintf(proxyuser, sizeof(proxyuser), "%s", g_getenv("http_proxy_user"));
-		else if (g_getenv("HTTPPROXYUSER"))
-			g_snprintf(proxyuser, sizeof(proxyuser), "%s", g_getenv("HTTPPROXYUSER"));
+		if (getVars)
+		{
+			if (g_getenv("HTTP_PROXY_PORT"))
+				proxyport = atoi(g_getenv("HTTP_PROXY_PORT"));
+			else if (g_getenv("http_proxy_port"))
+				proxyport = atoi(g_getenv("http_proxy_port"));
+			else if (g_getenv("HTTPPROXYPORT"))
+				proxyport = atoi(g_getenv("HTTPPROXYPORT"));
 
-		if (g_getenv("HTTP_PROXY_PASS"))
-			g_snprintf(proxypass, sizeof(proxypass), "%s", g_getenv("HTTP_PROXY_PASS"));
-		else if (g_getenv("http_proxy_pass"))
-			g_snprintf(proxypass, sizeof(proxypass), "%s", g_getenv("http_proxy_pass"));
-		else if (g_getenv("HTTPPROXYPASS"))
-			g_snprintf(proxypass, sizeof(proxypass), "%s", g_getenv("HTTPPROXYPASS"));
+			if (g_getenv("HTTP_PROXY_USER"))
+				g_snprintf(proxyuser, sizeof(proxyuser), "%s", g_getenv("HTTP_PROXY_USER"));
+			else if (g_getenv("http_proxy_user"))
+				g_snprintf(proxyuser, sizeof(proxyuser), "%s", g_getenv("http_proxy_user"));
+			else if (g_getenv("HTTPPROXYUSER"))
+				g_snprintf(proxyuser, sizeof(proxyuser), "%s", g_getenv("HTTPPROXYUSER"));
 
-		if (proxyhost[0])
-			proxytype = PROXY_HTTP;
+			if (g_getenv("HTTP_PROXY_PASS"))
+				g_snprintf(proxypass, sizeof(proxypass), "%s", g_getenv("HTTP_PROXY_PASS"));
+			else if (g_getenv("http_proxy_pass"))
+				g_snprintf(proxypass, sizeof(proxypass), "%s", g_getenv("http_proxy_pass"));
+			else if (g_getenv("HTTPPROXYPASS"))
+				g_snprintf(proxypass, sizeof(proxypass), "%s", g_getenv("HTTPPROXYPASS"));
+
+			
+			if (proxyhost[0])
+				proxytype = PROXY_HTTP;
+		}
 	}
 }
 
 static void gaimrc_write_proxy(FILE *f)
 {
+	char *str;
+
 	fprintf(f, "proxy {\n");
 	fprintf(f, "\thost { %s }\n", proxyhost);
 	fprintf(f, "\tport { %d }\n", proxyport);
 	fprintf(f, "\ttype { %d }\n", proxytype);
 	fprintf(f, "\tuser { %s }\n", proxyuser);
-	fprintf(f, "\tpass { %s }\n", proxypass);
+	fprintf(f, "\tpass { %s }\n", (str = escape_text2(proxypass)));
+	free(str);
 	fprintf(f, "}\n");
 }
 
@@ -1026,7 +1138,7 @@ static void set_defaults()
 	away_options =
 		OPT_AWAY_BACK_ON_IM;
 
-	for (i = 0; i < 7; i++)
+	for (i = 0; i < NUM_SOUNDS; i++)
 		sound_file[i] = NULL;
 	font_options = 0;
 	/* Enable all of the sound players that might be available.  The first
@@ -1048,7 +1160,7 @@ static void set_defaults()
 	report_idle = IDLE_GAIM;
 #endif
 	web_browser = BROWSER_NETSCAPE;
-	g_snprintf(web_command, sizeof(web_command), "xterm -e lynx \"%%s\"");
+	g_snprintf(web_command, sizeof(web_command), "xterm -e lynx %%s");
 
 	auto_away = 10;
 	a = g_new0(struct away_message, 1);
