@@ -366,7 +366,7 @@ void parse_toc_buddy_list(struct gaim_connection *gc, char *config)
 					how_many++;
 				}
 			} else if (*c == 'b' && !find_buddy(gc, c + 2)) {
-				char nm[80], sw[388], *tmp = c + 2;
+				char nm[80], sw[BUDDY_ALIAS_MAXLEN], *tmp = c + 2;
 				int i = 0;
 				while (*tmp != ':' && *tmp && i < sizeof(nm) - 1)
 					nm[i++] = *tmp++;
@@ -799,7 +799,7 @@ void do_export(struct gaim_connection *g)
 		toc_build_config(g, buf, 8192 - 1, TRUE);
 		fprintf(f, "%s\n", buf);
 		fclose(f);
-		chmod(buf, S_IRUSR | S_IWUSR);
+		chmod(path, S_IRUSR | S_IWUSR);
 	} else {
 		debug_printf("unable to write %s\n", path);
 	}
