@@ -68,7 +68,7 @@
 #define MIME_HEADER	"MIME-Version: 1.0\r\n" \
 			"Content-Type: text/plain; charset=UTF-8\r\n" \
 			"User-Agent: Gaim/" VERSION "\r\n" \
-			"X-MMS-IM-Format: FN=MS%20Sans%20Serif; EF=; CO=0; PF=0\r\n\r\n"
+			"X-MMS-IM-Format: FN=Arial; EF=; CO=0; PF=0\r\n\r\n"
 
 #define HOTMAIL_URL "http://www.hotmail.com/cgi-bin/folders"
 #define PASSPORT_URL "http://lc1.law13.hotmail.passport.com/cgi-bin/dologin?login="
@@ -85,6 +85,11 @@
 
 #define USEROPT_HOTMAIL 0
 
+#define USEROPT_MSNSERVER 3
+#define MSN_SERVER "messenger.hotmail.com"
+#define USEROPT_MSNPORT 4
+#define MSN_PORT 1863
+
 #define MSN_TYPING_RECV_TIMEOUT 6
 #define MSN_TYPING_SEND_TIMEOUT	4
 
@@ -157,9 +162,10 @@ static char *msn_normalize(const char *s)
 {
 	static char buf[BUF_LEN];
 
+	gchar * ss = g_strstrip(s);
 	g_return_val_if_fail(s != NULL, NULL);
 
-	g_snprintf(buf, sizeof(buf), "%s%s", s, strchr(s, '@') ? "" : "@hotmail.com");
+	g_snprintf(buf, sizeof(buf), "%s%s", ss, strchr(ss, '@') ? "" : "@hotmail.com");
 
 	return buf;
 }
@@ -222,128 +228,128 @@ static char *handle_errcode(char *buf, gboolean show)
 
 	switch (errcode) {
 		case 200:
-			g_snprintf(msg, sizeof(msg), "Syntax Error (probably a Gaim bug)");
+			g_snprintf(msg, sizeof(msg), _("Syntax Error (probably a Gaim bug)"));
 			break;
 		case 201:
-			g_snprintf(msg, sizeof(msg), "Invalid Parameter (probably a Gaim bug)");
+			g_snprintf(msg, sizeof(msg), _("Invalid Parameter (probably a Gaim bug)"));
 			break;
 		case 205:
-			g_snprintf(msg, sizeof(msg), "Invalid User");
+			g_snprintf(msg, sizeof(msg), _("Invalid User"));
 			break;
 		case 206:
-			g_snprintf(msg, sizeof(msg), "Fully Qualified Domain Name missing");
+			g_snprintf(msg, sizeof(msg), _("Fully Qualified Domain Name missing"));
 			break;
 		case 207:
-			g_snprintf(msg, sizeof(msg), "Already Login");
+			g_snprintf(msg, sizeof(msg), _("Already Login"));
 			break;
 		case 208:
-			g_snprintf(msg, sizeof(msg), "Invalid Username");
+			g_snprintf(msg, sizeof(msg), _("Invalid Username"));
 			break;
 		case 209:
-			g_snprintf(msg, sizeof(msg), "Invalid Friendly Name");
+			g_snprintf(msg, sizeof(msg), _("Invalid Friendly Name"));
 			break;
 		case 210:
-			g_snprintf(msg, sizeof(msg), "List Full");
+			g_snprintf(msg, sizeof(msg), _("List Full"));
 			break;
 		case 215:
-			g_snprintf(msg, sizeof(msg), "Already there");
+			g_snprintf(msg, sizeof(msg), _("Already there"));
 			break;
 		case 216:
-			g_snprintf(msg, sizeof(msg), "Not on list");
+			g_snprintf(msg, sizeof(msg), _("Not on list"));
 			break;
 		case 217:
-			g_snprintf(msg, sizeof(msg), "User is offline");
+			g_snprintf(msg, sizeof(msg), _("User is offline"));
 			break;
 		case 218:
-			g_snprintf(msg, sizeof(msg), "Already in the mode");
+			g_snprintf(msg, sizeof(msg), _("Already in the mode"));
 			break;
 		case 219:
-			g_snprintf(msg, sizeof(msg), "Already in opposite list");
+			g_snprintf(msg, sizeof(msg), _("Already in opposite list"));
 			break;
 		case 280:
-			g_snprintf(msg, sizeof(msg), "Switchboard failed");
+			g_snprintf(msg, sizeof(msg), _("Switchboard failed"));
 			break;
 		case 281:
-			g_snprintf(msg, sizeof(msg), "Notify Transfer failed");
+			g_snprintf(msg, sizeof(msg), _("Notify Transfer failed"));
 			break;
 
 		case 300:
-			g_snprintf(msg, sizeof(msg), "Required fields missing");
+			g_snprintf(msg, sizeof(msg), _("Required fields missing"));
 			break;
 		case 302:
-			g_snprintf(msg, sizeof(msg), "Not logged in");
+			g_snprintf(msg, sizeof(msg), _("Not logged in"));
 			break;
 
 		case 500:
-			g_snprintf(msg, sizeof(msg), "Internal server error");
+			g_snprintf(msg, sizeof(msg), _("Internal server error"));
 			break;
 		case 501:
-			g_snprintf(msg, sizeof(msg), "Database server error");
+			g_snprintf(msg, sizeof(msg), _("Database server error"));
 			break;
 		case 510:
-			g_snprintf(msg, sizeof(msg), "File operation error");
+			g_snprintf(msg, sizeof(msg), _("File operation error"));
 			break;
 		case 520:
-			g_snprintf(msg, sizeof(msg), "Memory allocation error");
+			g_snprintf(msg, sizeof(msg), _("Memory allocation error"));
 			break;
 
 		case 600:
-			g_snprintf(msg, sizeof(msg), "Server busy");
+			g_snprintf(msg, sizeof(msg), _("Server busy"));
 			break;
 		case 601:
-			g_snprintf(msg, sizeof(msg), "Server unavailable");
+			g_snprintf(msg, sizeof(msg), _("Server unavailable"));
 			break;
 		case 602:
-			g_snprintf(msg, sizeof(msg), "Peer Notification server down");
+			g_snprintf(msg, sizeof(msg), _("Peer Notification server down"));
 			break;
 		case 603:
-			g_snprintf(msg, sizeof(msg), "Database connect error");
+			g_snprintf(msg, sizeof(msg), _("Database connect error"));
 			break;
 		case 604:
-			g_snprintf(msg, sizeof(msg), "Server is going down (abandon ship)");
+			g_snprintf(msg, sizeof(msg), _("Server is going down (abandon ship)"));
 			break;
 
 		case 707:
-			g_snprintf(msg, sizeof(msg), "Error creating connection");
+			g_snprintf(msg, sizeof(msg), _("Error creating connection"));
 			break;
 		case 711:
-			g_snprintf(msg, sizeof(msg), "Unable to write");
+			g_snprintf(msg, sizeof(msg), _("Unable to write"));
 			break;
 		case 712:
-			g_snprintf(msg, sizeof(msg), "Session overload");
+			g_snprintf(msg, sizeof(msg), _("Session overload"));
 			break;
 		case 713:
-			g_snprintf(msg, sizeof(msg), "User is too active");
+			g_snprintf(msg, sizeof(msg), _("User is too active"));
 			break;
 		case 714:
-			g_snprintf(msg, sizeof(msg), "Too many sessions");
+			g_snprintf(msg, sizeof(msg), _("Too many sessions"));
 			break;
 		case 715:
-			g_snprintf(msg, sizeof(msg), "Not expected");
+			g_snprintf(msg, sizeof(msg), _("Not expected"));
 			break;
 		case 717:
-			g_snprintf(msg, sizeof(msg), "Bad friend file");
+			g_snprintf(msg, sizeof(msg), _("Bad friend file"));
 			break;
 
 		case 911:
-			g_snprintf(msg, sizeof(msg), "Authentication failed");
+			g_snprintf(msg, sizeof(msg), _("Authentication failed"));
 			break;
 		case 913:
-			g_snprintf(msg, sizeof(msg), "Not allowed when offline");
+			g_snprintf(msg, sizeof(msg), _("Not allowed when offline"));
 			break;
 	        case 920:
-			g_snprintf(msg, sizeof(msg), "Not accepting new users");
+			g_snprintf(msg, sizeof(msg), _("Not accepting new users"));
 			break;
 	        case 924:
-			g_snprintf(msg, sizeof(msg), "User unverified");
+			g_snprintf(msg, sizeof(msg), _("User unverified"));
 			break;
 		default:
-			g_snprintf(msg, sizeof(msg), "Unknown Error Code");
+			g_snprintf(msg, sizeof(msg), _("Unknown Error Code"));
 			break;
 	}
 
 	if (show)
-		do_error_dialog(msg, "MSN Error");
+		do_error_dialog(msg, _("MSN Error"));
 
 	return msg;
 }
@@ -354,6 +360,10 @@ static void handle_hotmail(struct gaim_connection *gc, char *data)
 	char buf[MSN_BUF_LEN];
 	struct msn_data *md = gc->proto_data;
 
+	if (strchr(gc->username, '@') != strstr(gc->username, "@hotmail.com"))
+		/* We can only get Hotmail notification from hotmail users */
+		return;
+
 	if (!md->passport) {
 		g_snprintf(buf, sizeof(buf), "URL %d INBOX\r\n", ++md->trId);
 
@@ -380,7 +390,9 @@ static void handle_hotmail(struct gaim_connection *gc, char *data)
 			x = strstr(from, "\r\n"); *x = 0;
 			subject += strlen("Subject: ");
 			x = strstr(subject, "\r\n"); *x = 0;
+			from = utf8_to_str(from);
 			connection_has_mail(gc, -1, from, subject, login_url);
+			g_free(from);
 		}
 	}
 }
@@ -544,7 +556,7 @@ static int msn_process_switch(struct msn_switchboard *ms, char *buf)
 		ms->msguser = g_strdup(user);
 		ms->msglen = length;
 	} else if (!g_strncasecmp(buf, "NAK", 3)) {
-		do_error_dialog("A message may not have been received.", "MSN Error");
+		do_error_dialog(_("A message may not have been received."), _("MSN Error"));
 	} else if (!g_strncasecmp(buf, "NLN", 3)) {
 	} else if (!g_strncasecmp(buf, "OUT", 3)) {
 		if (ms->chat)
@@ -572,28 +584,6 @@ static int msn_process_switch(struct msn_switchboard *ms, char *buf)
 	return 1;
 }
 
-static void msn_unescape(char *text) {
-	char *cpy = g_strdup(text);
-	char *cur = cpy;
-	int c = 0;
-
-
-	while (*cur) {
-		if (*cur == '%') {
-			if (sscanf (cur, "%%%x;", &c) == 1 && c != 0) {
-				*text = c;
-				cur = cur + 3;
-			} 
-		} else {
-			*text = *cur;
-			cur++;
-		}
-		text++;
-	}
-	*text = 0;
-	g_free(cpy);
-}
-
 static char *msn_parse_format(char *mime)
 {
 	char *cur;
@@ -629,9 +619,8 @@ static char *msn_parse_format(char *mime)
 		}
 	}
 	
-	msn_unescape(ret->str);
-	cur = ret->str;
-	g_string_free(ret, FALSE);
+	cur = url_decode(ret->str);
+	g_string_free(ret, TRUE);
 	return cur;
 }
 			
@@ -660,7 +649,7 @@ static void msn_process_switch_msg(struct msn_switchboard *ms, char *msg)
 		return;
 	if (!g_strncasecmp(content, "Content-Type: text/x-msmsgscontrol\r\n",
 			   strlen(  "Content-Type: text/x-msmsgscontrol\r\n"))) {
-		if (strstr(content,"TypingUser: ")) {
+		if (strstr(content,"TypingUser: ") && !ms->chat) {
 			serv_got_typing(ms->gc, ms->msguser, MSN_TYPING_RECV_TIMEOUT);
 			return;
 		} 
@@ -694,7 +683,6 @@ static void msn_process_switch_msg(struct msn_switchboard *ms, char *msg)
 
 		g_free(message);
 		if (format) {
-			g_free(format);
 			g_free(utf);
 		}
 	}
@@ -851,8 +839,12 @@ static void msn_accept_add(gpointer w, struct msn_add_permit *map)
 {
 	struct msn_data *md = map->gc->proto_data;
 	char buf[MSN_BUF_LEN];
-
-	g_snprintf(buf, sizeof(buf), "ADD %d AL %s %s\r\n", ++md->trId, map->user, url_encode(map->friend));
+	char *srvfriend;
+	
+	srvfriend = str_to_utf8(map->friend);
+	g_snprintf(buf, sizeof(buf), "ADD %d AL %s %s\r\n", ++md->trId, map->user, url_encode(srvfriend));
+	g_free(srvfriend);
+	
 	if (msn_write(md->fd, buf, strlen(buf)) < 0) {
 		hide_login_progress(map->gc, "Write error");
 		signoff(map->gc);
@@ -868,9 +860,10 @@ static void msn_cancel_add(gpointer w, struct msn_add_permit *map)
 {
 	struct msn_data *md = map->gc->proto_data;
 	char buf[MSN_BUF_LEN];
+	char *srvfriend = str_to_utf8(map->friend);
 
 	if (*(map->user)) {
-		g_snprintf(buf, sizeof(buf), "ADD %d BL %s %s\r\n", ++md->trId, map->user, url_encode(map->friend));
+		g_snprintf(buf, sizeof(buf), "ADD %d BL %s %s\r\n", ++md->trId, map->user, url_encode(srvfriend));
 		if (msn_write(md->fd, buf, strlen(buf)) < 0) {
 			hide_login_progress(map->gc, "Write error");
 			signoff(map->gc);
@@ -880,6 +873,7 @@ static void msn_cancel_add(gpointer w, struct msn_add_permit *map)
 		build_block_list();
 	}
 	
+	g_free(srvfriend);
 	g_free(map->user);
 	g_free(map->friend);
 	g_free(map);
@@ -905,7 +899,7 @@ static int msn_process_main(struct gaim_connection *gc, char *buf)
 		user = tmp;
 
 		GET_NEXT(tmp);
-		friend = tmp;
+		friend = url_decode(tmp);
 
 		if (g_strcasecmp(list, "RL"))
 			return 1;
@@ -918,11 +912,11 @@ static int msn_process_main(struct gaim_connection *gc, char *buf)
 
 		ap = g_new0(struct msn_add_permit, 1);
 		ap->user = g_strdup(user);
-		ap->friend = g_strdup(url_decode(friend));
+		ap->friend = utf8_to_str(friend);
 		ap->gc = gc;
 
-		g_snprintf(msg, sizeof(msg), "The user %s (%s) wants to add you to their buddy list.",
-				ap->user, url_decode(ap->friend));
+		g_snprintf(msg, sizeof(msg), _("The user %s (%s) wants to add %s to his or her buddy list."),
+				ap->user, ap->friend, ap->gc->username);
 
 		do_ask_dialog(msg, ap, msn_accept_add, msn_cancel_add);
 	} else if (!g_strncasecmp(buf, "BLP", 3)) {
@@ -1002,7 +996,7 @@ static int msn_process_main(struct gaim_connection *gc, char *buf)
 		if ((b = find_buddy(gc, user)) != NULL) {
 			if (b->proto_data)
 				g_free(b->proto_data);
-			b->proto_data = g_strdup(friend);
+			b->proto_data = utf8_to_str(friend);
 		}
 
 		if (!g_strcasecmp(state, "BSY")) {
@@ -1047,9 +1041,15 @@ static int msn_process_main(struct gaim_connection *gc, char *buf)
 		if (!g_strcasecmp(which, "FL") && pos) {
 			struct msn_buddy *b = g_new0(struct msn_buddy, 1);
 			b->user = g_strdup(who);
-			b->friend = g_strdup(friend);
+			b->friend = utf8_to_str(friend);
 			md->fl = g_slist_append(md->fl, b);
 		} else if (!g_strcasecmp(which, "AL") && pos) {
+			char *dupl;
+			if ((dupl = (char *)g_slist_find_custom(gc->deny, who, 
+							(GCompareFunc)strcmp))) {
+				debug_printf("moving from deny to permit: %s", who);
+				gc->deny = g_slist_remove(gc->deny, dupl);
+			}
 			gc->permit = g_slist_append(gc->permit, g_strdup(who));
 		} else if (!g_strcasecmp(which, "BL") && pos) {
 			gc->deny = g_slist_append(gc->deny, g_strdup(who));
@@ -1069,10 +1069,10 @@ static int msn_process_main(struct gaim_connection *gc, char *buf)
 				debug_printf("Unresolved MSN RL entry\n");
 				ap = g_new0(struct msn_add_permit, 1);
 				ap->user = g_strdup(who);
-				ap->friend = g_strdup(friend);
+				ap->friend = utf8_to_str(friend);
 				ap->gc = gc;
                          
-		                g_snprintf(msg, sizeof(msg), "The user %s (%s) wants to add you to their buddy list",ap->user, url_decode(ap->friend));
+		                g_snprintf(msg, sizeof(msg), _("The user %s (%s) wants to add you to their buddy list"),ap->user, ap->friend);
 				do_ask_dialog(msg, ap, msn_accept_add, msn_cancel_add);
 			}
 		    }
@@ -1108,7 +1108,7 @@ static int msn_process_main(struct gaim_connection *gc, char *buf)
 				struct buddy *b;
 				md->fl = g_slist_remove(md->fl, mb);
 				if (!(b = find_buddy(gc, mb->user)))
-					add_buddy(gc, "Buddies", mb->user, mb->friend);
+					add_buddy(gc, _("Buddies"), mb->user, mb->friend);
 				else if (!g_strcasecmp(b->name, b->show)) {
 					g_snprintf(b->show, sizeof(b->show), "%s", mb->friend);
 					handle_buddy_rename(b, b->name);
@@ -1150,7 +1150,7 @@ static int msn_process_main(struct gaim_connection *gc, char *buf)
 		if ((b = find_buddy(gc, user)) != NULL) {
 			if (b->proto_data)
 				g_free(b->proto_data);
-			b->proto_data = g_strdup(friend);
+			b->proto_data = utf8_to_str(friend);
 		}
 
 		if (!g_strcasecmp(state, "BSY")) {
@@ -1188,9 +1188,12 @@ static int msn_process_main(struct gaim_connection *gc, char *buf)
 		GET_NEXT(tmp);
 		GET_NEXT(tmp);
 		GET_NEXT(tmp);
+
 		friend = url_decode(tmp);
+		friend = utf8_to_str(friend);
 
 		g_snprintf(gc->displayname, sizeof(gc->displayname), "%s", friend);
+		g_free(friend);
 	} else if (!g_strncasecmp(buf, "REM", 3)) {
 	} else if (!g_strncasecmp(buf, "RNG", 3)) {
 		struct msn_switchboard *ms;
@@ -1242,7 +1245,6 @@ static int msn_process_main(struct gaim_connection *gc, char *buf)
 		char hippy[2048];
 		char *rru;
 		char *passport;
-		char *filename;
 
 		GET_NEXT(tmp);
 		GET_NEXT(tmp);
@@ -1250,7 +1252,7 @@ static int msn_process_main(struct gaim_connection *gc, char *buf)
 		GET_NEXT(tmp);
 		passport = tmp;
 		
-		snprintf(hippy, sizeof(hippy), "%s%d%s", md->mspauth, time(NULL) - md->sl, gc->password);
+		snprintf(hippy, sizeof(hippy), "%s%lu%s", md->mspauth, time(NULL) - md->sl, gc->password);
 
 		md5_init(&st);
 		md5_append(&st, (const md5_byte_t *)hippy, strlen(hippy));
@@ -1433,7 +1435,7 @@ static void msn_callback(gpointer data, gint source, GaimInputCondition cond)
 	len = read(md->fd, buf, sizeof(buf));
 
 	if (len <= 0) {
-		hide_login_progress_error(gc, "Error reading from server");
+		hide_login_progress_error(gc, _("Error reading from server"));
 		signoff(gc);
 		return;
 	}
@@ -1566,7 +1568,7 @@ static int msn_process_login(struct gaim_connection *gc, char *buf)
 			return 0;
 		}
 
-		set_login_progress(gc, 3, "Requesting to send password");
+		set_login_progress(gc, 3, _("Requesting to send password"));
 	} else if (!g_strncasecmp(buf, "USR", 3)) {
 		char *resp, *friend, *tmp = buf;
 
@@ -1575,13 +1577,14 @@ static int msn_process_login(struct gaim_connection *gc, char *buf)
 		resp = tmp;
 		GET_NEXT(tmp);
 		GET_NEXT(tmp);
-		friend = tmp;
+		friend = url_decode(tmp);
 		GET_NEXT(tmp);
-		friend = url_decode(friend);
 
 		/* so here, we're either getting the challenge or the OK */
 		if (!g_strcasecmp(resp, "OK")) {
+			friend = utf8_to_str(friend);
 			g_snprintf(gc->displayname, sizeof(gc->displayname), "%s", friend);
+			g_free(friend);
 
 			g_snprintf(sendbuf, sizeof(sendbuf), "SYN %d 0\r\n", ++md->trId);
 			if (msn_write(md->fd, sendbuf, strlen(sendbuf)) < 0) {
@@ -1599,7 +1602,9 @@ static int msn_process_login(struct gaim_connection *gc, char *buf)
 			md5_byte_t di[16];
 			int i;
 
+			friend = utf8_to_str(friend);
 			g_snprintf(buf2, sizeof(buf2), "%s%s", friend, gc->password);
+			g_free(friend);
 
 			md5_init(&st);
 			md5_append(&st, (const md5_byte_t *)buf2, strlen(buf2));
@@ -1610,15 +1615,15 @@ static int msn_process_login(struct gaim_connection *gc, char *buf)
 				g_snprintf(buf2, sizeof(buf2), "%02x", di[i]);
 				strcat(sendbuf, buf2);
 			}
-			strcat(sendbuf, "\n");
+			strcat(sendbuf, "\r\n");
 
 			if (msn_write(md->fd, sendbuf, strlen(sendbuf)) < 0) {
-				hide_login_progress(gc, "Unable to send password");
+				hide_login_progress(gc, _("Unable to send password"));
 				signoff(gc);
 				return 0;
 			}
 
-			set_login_progress(gc, 4, "Password sent");
+			set_login_progress(gc, 4, _("Password sent"));
 		}
 	} else if (!g_strncasecmp(buf, "XFR", 3)) {
 		char *host = strstr(buf, "NS");
@@ -1674,7 +1679,7 @@ static void msn_login_callback(gpointer data, gint source, GaimInputCondition co
 	len = read(md->fd, buf, sizeof(buf));
 
 	if (len <= 0) {
-		hide_login_progress(gc, "Error reading from server");
+		hide_login_progress(gc, _("Error reading from server"));
 		signoff(gc);
 		return;
 	}
@@ -1736,20 +1741,20 @@ static void msn_login_connect(gpointer data, gint source, GaimInputCondition con
 		md->fd = source;
 
 	if (md->fd == -1) {
-		hide_login_progress(gc, "Unable to connect");
+		hide_login_progress(gc, _("Unable to connect"));
 		signoff(gc);
 		return;
 	}
 
 	g_snprintf(buf, sizeof(buf), "VER %d MSNP5\r\n", ++md->trId);
 	if (msn_write(md->fd, buf, strlen(buf)) < 0) {
-		hide_login_progress(gc, "Unable to write to server");
+		hide_login_progress(gc, _("Unable to write to server"));
 		signoff(gc);
 		return;
 	}
 
 	md->inpa = gaim_input_add(md->fd, GAIM_INPUT_READ, msn_login_callback, gc);
-	set_login_progress(gc, 2, "Synching with server");
+	set_login_progress(gc, 2,_("Synching with server"));
 }
 
 static void msn_login(struct aim_user *user)
@@ -1757,13 +1762,15 @@ static void msn_login(struct aim_user *user)
 	struct gaim_connection *gc = new_gaim_conn(user);
 	struct msn_data *md = gc->proto_data = g_new0(struct msn_data, 1);
 
-	set_login_progress(gc, 1, "Connecting");
+	set_login_progress(gc, 1, _("Connecting"));
 
 	g_snprintf(gc->username, sizeof(gc->username), "%s", msn_normalize(gc->username));
-
-	md->fd = proxy_connect("messenger.hotmail.com", 1863, msn_login_connect, gc);
+	
+	md->fd = proxy_connect(user->proto_opt[USEROPT_MSNSERVER][0] ? user->proto_opt[USEROPT_MSNSERVER] : MSN_SERVER, 
+			       user->proto_opt[USEROPT_MSNPORT][0] ? atoi(user->proto_opt[USEROPT_MSNPORT]) : MSN_PORT,
+			       msn_login_connect, gc);
 	if (md->fd < 0) {
-		hide_login_progress(gc, "Unable to connect");
+		hide_login_progress(gc, _("Unable to connect"));
 		signoff(gc);
 	}
 }
@@ -1912,13 +1919,13 @@ static GList *msn_away_states(struct gaim_connection *gc)
 {
 	GList *m = NULL;
 
-	m = g_list_append(m, "Available");
-	m = g_list_append(m, "Away From Computer");
-	m = g_list_append(m, "Be Right Back");
-	m = g_list_append(m, "Busy");
-	m = g_list_append(m, "On The Phone");
-	m = g_list_append(m, "Out To Lunch");
-	m = g_list_append(m, "Hidden");
+	m = g_list_append(m, _("Available"));
+	m = g_list_append(m, _("Away From Computer"));
+	m = g_list_append(m, _("Be Right Back"));
+	m = g_list_append(m, _("Busy"));
+	m = g_list_append(m, _("On The Phone"));
+	m = g_list_append(m, _("Out To Lunch"));
+	m = g_list_append(m, _("Hidden"));
 
 	return m;
 }
@@ -2034,7 +2041,7 @@ static GList *msn_buddy_menu(struct gaim_connection *gc, char *who)
 	static char buf[MSN_BUF_LEN];
 
 	pbm = g_new0(struct proto_buddy_menu, 1);
-	pbm->label = "Reset friendly name";
+	pbm->label = _("Reset friendly name");
 	pbm->callback = msn_reset_friend;
 	pbm->gc = gc;
 	m = g_list_append(m, pbm);
@@ -2043,7 +2050,7 @@ static GList *msn_buddy_menu(struct gaim_connection *gc, char *who)
 		return m;
 
 	pbm = g_new0(struct proto_buddy_menu, 1);
-	g_snprintf(buf, sizeof(buf), "Status: %s", msn_get_away_text(b->uc >> 1));
+	g_snprintf(buf, sizeof(buf), _("Status: %s"), msn_get_away_text(b->uc >> 1));
 	pbm->label = buf;
 	pbm->callback = NULL;
 	pbm->gc = gc;
@@ -2102,13 +2109,20 @@ static void msn_act_id(gpointer data, char *entry)
 	struct gaim_connection *gc = data;
 	struct msn_data *md = gc->proto_data;
 	char buf[MSN_BUF_LEN];
+	char *alias;
 
-	if (strlen(url_encode(entry)) >= BUDDY_ALIAS_MAXLEN) {
-		do_error_dialog("Friendly name too long.", "MSN Error");
+	if (!entry || *entry == 0)
+		alias = g_strdup("");
+	else
+		alias = str_to_utf8(entry);
+	
+	if (strlen(alias) >= BUDDY_ALIAS_MAXLEN) {
+		do_error_dialog(_("Friendly name too long."), _("MSN Error"));
 		return;
 	}
 	
-	g_snprintf(buf, sizeof(buf), "REA %d %s %s\r\n", ++md->trId, gc->username, url_encode(entry));
+	g_snprintf(buf, sizeof(buf), "REA %d %s %s\r\n", ++md->trId, gc->username, url_encode(alias));
+	g_free(alias);
 	if (msn_write(md->fd, buf, strlen(buf)) < 0) {
 		hide_login_progress(gc, "Write error");
 		signoff(gc);
@@ -2118,9 +2132,9 @@ static void msn_act_id(gpointer data, char *entry)
 
 static void msn_do_action(struct gaim_connection *gc, char *act)
 {
-	if (!strcmp(act, "Set Friendly Name")) {
-		do_prompt_dialog("Set Friendly Name:", gc->displayname, gc, msn_act_id, NULL);
-	} else if (!strcmp(act, "Reset All Friendly Names")) {
+	if (!strcmp(act, _("Set Friendly Name"))) {
+		do_prompt_dialog(_("Set Friendly Name:"), gc->displayname, gc, msn_act_id, NULL);
+	} else if (!strcmp(act, _("Reset All Friendly Names"))) {
 		GSList *g = gc->groups;
 		while (g) {
 			GSList *m = ((struct group *)g->data)->members;
@@ -2139,7 +2153,7 @@ static GList *msn_actions()
 {
 	GList *m = NULL;
 
-	m = g_list_append(m, "Set Friendly Name");
+	m = g_list_append(m, _("Set Friendly Name"));
 	m = g_list_append(m, "Reset All Friendly Names");
 
 	return m;
@@ -2201,11 +2215,16 @@ static void msn_set_permit_deny(struct gaim_connection *gc)
 		s = g_slist_nth(gc->permit, g_slist_length(md->permit));
 		while (s) {
 			char *who = s->data;
+			char *dupl;
 			s = s->next;
 			if (!strchr(who, '@')) {
 				t = g_slist_append(t, who);
 				continue;
 			}
+			if ((dupl = (char *)g_slist_find(md->deny, who))) {
+				t = g_slist_append(t, who);
+				continue;
+			}
 			g_snprintf(buf, sizeof(buf), "ADD %d AL %s %s\r\n", ++md->trId, who, who);
 			if (msn_write(md->fd, buf, strlen(buf)) < 0) {
 				hide_login_progress(gc, "Write error");
@@ -2230,11 +2249,16 @@ static void msn_set_permit_deny(struct gaim_connection *gc)
 		s = g_slist_nth(gc->deny, g_slist_length(md->deny));
 		while (s) {
 			char *who = s->data;
+			char *dupl;
 			s = s->next;
 			if (!strchr(who, '@')) {
 				t = g_slist_append(t, who);
 				continue;
 			}
+			if ((dupl = (char *)g_slist_find(md->deny, who))) {
+				t = g_slist_append(t, who);
+				continue;
+			}
 			g_snprintf(buf, sizeof(buf), "ADD %d BL %s %s\r\n", ++md->trId, who, who);
 			if (msn_write(md->fd, buf, strlen(buf)) < 0) {
 				hide_login_progress(gc, "Write error");
@@ -2259,6 +2283,7 @@ static void msn_add_permit(struct gaim_connection *gc, char *who)
 {
 	struct msn_data *md = gc->proto_data;
 	char buf[MSN_BUF_LEN];
+	char *dupl;
 
 	if (!strchr(who, '@')) {
 		do_error_dialog(_("Invalid name"), _("MSN Error"));
@@ -2267,6 +2292,17 @@ static void msn_add_permit(struct gaim_connection *gc, char *who)
 		return;
 	}
 
+	if ((dupl = (char *)g_slist_find_custom(gc->deny, who, 
+					(GCompareFunc)strcmp))) {
+		debug_printf("MSN: Moving %s from BL to AL\n", who);
+		gc->deny = g_slist_remove(gc->deny, dupl);
+		g_snprintf(buf, sizeof(buf), "REM %d BL %s\r\n", ++md->trId, who);
+			if (msn_write(md->fd, buf, strlen(buf)) < 0) {
+				hide_login_progress(gc, "Write error");
+				signoff(gc);
+				return;
+			}
+	}
 	g_snprintf(buf, sizeof(buf), "ADD %d AL %s %s\r\n", ++md->trId, who, who);
 	if (msn_write(md->fd, buf, strlen(buf)) < 0) {
 		hide_login_progress(gc, "Write error");
@@ -2286,13 +2322,22 @@ static void msn_rem_permit(struct gaim_connection *gc, char *who)
 		signoff(gc);
 		return;
 	}
+	
+	g_slist_append(gc->deny, who);
+	g_snprintf(buf, sizeof(buf), "ADD %d BL %s %s\r\n", ++md->trId, who, who);
+	if (msn_write(md->fd, buf, strlen(buf)) < 0) {
+		hide_login_progress(gc, "Write error");
+		signoff(gc);
+		return;
+	}
 }
 
 static void msn_add_deny(struct gaim_connection *gc, char *who)
 {
 	struct msn_data *md = gc->proto_data;
 	char buf[MSN_BUF_LEN];
-
+	char *dupl;
+	
 	if (!strchr(who, '@')) {
 		do_error_dialog(_("Invalid name"), _("MSN Error"));
 		gc->deny = g_slist_remove(gc->deny, who);
@@ -2300,6 +2345,19 @@ static void msn_add_deny(struct gaim_connection *gc, char *who)
 		return;
 	}
 
+	if ((dupl = (char *)g_slist_find_custom(gc->permit, who, 
+					(GCompareFunc)strcmp))) {
+		debug_printf("MSN: Moving %s from AL to BL\n", who);
+		gc->permit = g_slist_remove(gc->permit, dupl);
+		g_snprintf(buf, sizeof(buf), "REM %d AL %s\r\n", ++md->trId, who);
+		if (msn_write(md->fd, buf, strlen(buf)) < 0) {
+			hide_login_progress(gc, "Write error");
+			signoff(gc);
+			return;
+		}
+	}
+		
+
 	g_snprintf(buf, sizeof(buf), "ADD %d BL %s %s\r\n", ++md->trId, who, who);
 	if (msn_write(md->fd, buf, strlen(buf)) < 0) {
 		hide_login_progress(gc, "Write error");
@@ -2319,6 +2377,14 @@ static void msn_rem_deny(struct gaim_connection *gc, char *who)
 		signoff(gc);
 		return;
 	}
+	
+	g_slist_append(gc->permit, who);
+	g_snprintf(buf, sizeof(buf), "ADD %d AL %s %s\r\n", ++md->trId, who, who);
+	if (msn_write(md->fd, buf, strlen(buf)) < 0) {
+		hide_login_progress(gc, "Write error");
+		signoff(gc);
+		return;
+	}
 }
 
 static void msn_buddy_free(struct buddy *b)
@@ -2327,6 +2393,26 @@ static void msn_buddy_free(struct buddy *b)
 		g_free(b->proto_data);
 }
 
+static GList *msn_user_opts()
+{
+	GList *m = NULL;
+	struct proto_user_opt *puo;
+
+	puo = g_new0(struct proto_user_opt, 1);
+	puo->label = "Server:";
+	puo->def = MSN_SERVER;
+	puo->pos = USEROPT_MSNSERVER;
+	m = g_list_append(m, puo);
+
+	puo = g_new0(struct proto_user_opt, 1);
+	puo->label = "Port:";
+	puo->def = "1863";
+	puo->pos = USEROPT_MSNPORT;
+	m = g_list_append(m, puo);
+
+	return m;
+}
+
 GSList *msn_smiley_list() 
 { 
 	GSList *smilies = NULL;
@@ -2462,6 +2548,7 @@ void msn_init(struct prpl *ret)
 	ret->rem_deny = msn_rem_deny;
 	ret->buddy_free = msn_buddy_free;
 	ret->smiley_list = msn_smiley_list;
+	ret->user_opts = msn_user_opts;
 
 	my_protocol = ret;
 }
