@@ -428,7 +428,8 @@ void connection_has_mail(struct gaim_connection *gc, int count, const char *from
 
 	mn->email_label = gtk_label_new(buf);
 	gtk_label_set_text(GTK_LABEL(mn->email_label), buf);
-	gtk_box_pack_start(GTK_BOX(vbox), mn->email_label, 0, 0, 5);
+	gtk_label_set_line_wrap(GTK_LABEL(mn->email_label), TRUE);
+	gtk_box_pack_start(GTK_BOX(vbox), mn->email_label, FALSE, TRUE, 5);
 	gtk_widget_show(mn->email_label);
 
 	hbox = gtk_hbox_new(FALSE, 5);
@@ -470,9 +471,15 @@ static gint find_icon_data(gconstpointer a, gconstpointer b)
 
 void set_icon_data(struct gaim_connection *gc, char *who, void *data, int len)
 {
-	struct icon_data tmp = { gc, normalize(who), NULL, 0 };
-	GList *l = g_list_find_custom(icons, &tmp, find_icon_data);
-	struct icon_data *id = l ? l->data : NULL;
+	struct icon_data tmp;
+	GList *l;
+	struct icon_data *id;
+	tmp.gc = gc;
+	tmp.who = normalize(who);
+	tmp.data=NULL;
+        tmp.len = 0;
+	l = g_list_find_custom(icons, &tmp, find_icon_data);
+	id = l ? l->data : NULL;
 
 	if (id) {
 		g_free(id->data);
@@ -561,7 +568,7 @@ void show_got_added(struct gaim_connection *gc, const char *id,
 	ga->who = g_strdup(who);
 	ga->alias = alias ? g_strdup(alias) : NULL;
 
-	g_snprintf(buf, sizeof(buf), _("%s%s%s%s has made %s their buddy%s%s%s"),
+	g_snprintf(buf, sizeof(buf), _("%s%s%s%s has made %s his or her buddy%s%s%s"),
 		   who,
 		   alias ? " (" : "",
 		   alias ? alias : "",
@@ -569,7 +576,7 @@ void show_got_added(struct gaim_connection *gc, const char *id,
 		   id ? id : gc->displayname[0] ? gc->displayname : gc->username,
 		   msg ? ": " : ".",
 		   msg ? msg : "",
-		   find_buddy(gc, ga->who) ? "" : _("\n\nDo you wish to add them to your buddy list?"));
+		   find_buddy(gc, ga->who) ? "" : _("\n\nDo you wish to add him or her to your buddy list?"));
 	if (find_buddy(gc, ga->who))
 		do_error_dialog(buf, "Added to List");
 	else
@@ -690,3 +697,16 @@ void register_dialog()
 
 	gtk_widget_show(regdlg);
 }
+
+GSList *add_smiley(GSList *list, char *key, char **xpm, int show) 
+{
+	struct _prpl_smiley *smiley;
+
+ 	smiley = (struct _prpl_smiley *)g_new0(struct _prpl_smiley, 1);
+	smiley->key = g_strdup(key);
+	smiley->xpm = xpm;
+	smiley->show = show;
+	list = g_slist_append(list, smiley);
+
+	return list;
+}
