@@ -32,6 +32,9 @@
 #include <string.h>
 #include <sys/wait.h>
 #include <ctype.h>
+#ifdef HAVE_ICONV
+#include <iconv.h>
+#endif
 #include <math.h>
 #include "gaim.h"
 #include "prpl.h"
@@ -252,7 +255,7 @@ gint linkify_text(char *text)
 			char *tmp;
 			int flag;
 			int len = 0;
-			char illegal_chars[] = "!@#$%^&*()[]{}/\\<>\":;\0";
+			char illegal_chars[] = "!@#$%^&*()[]{}/|\\<>\":;\0";
 			url_buf[0] = 0;
 
 			if (*(c - 1) == ' ' || *(c + 1) == ' ' || rindex(illegal_chars, *(c + 1))
@@ -283,15 +286,15 @@ gint linkify_text(char *text)
 				}
 			}
 
-
 			t = c + 1;
 
 			while (flag) {
 				if (badchar(*t)) {
-					if (*(t - 1) == '.') {
-						url_buf[t-c] = '\0';
-						t--;
-					}
+					char *d;
+
+					for (d = url_buf + strlen(url_buf) - 1; *d == '.'; d--, t--)
+						*d = '\0';
+
 					cnt += g_snprintf(&text[cnt], 1024,
 							  "<A HREF=\"mailto:%s\">%s</A>", url_buf,
 							  url_buf);
@@ -308,10 +311,7 @@ gint linkify_text(char *text)
 				}
 
 				t++;
-
 			}
-
-
 		}
 
 		if (*c == 0)
@@ -928,7 +928,7 @@ void strncpy_withhtml(gchar *dest, const gchar *src, size_t destsize)
 
 gchar *strdup_withhtml(const gchar *src)
 {
-	char *sp, *dest;
+	gchar *sp, *dest;
 	gulong destsize;
 
 	/*
@@ -936,7 +936,7 @@ gchar *strdup_withhtml(const gchar *src)
 	 * additional length of "<BR>" over "\n"), account for the
 	 * terminator, malloc the space and call strncpy_withhtml.
 	 */
-	for(destsize = 0, sp = src; (sp = strchr(sp, '\n')) != NULL; ++sp, ++destsize)
+	for(destsize = 0, sp = (gchar *)src; (sp = strchr(sp, '\n')) != NULL; ++sp, ++destsize)
 		;
 	destsize *= 3;
 	destsize += strlen(src) + 1;
@@ -1121,92 +1121,49 @@ void system_log(enum log_event what, struct gaim_connection *gc, struct buddy *w
 	fclose(fd);
 }
 
-unsigned char *utf8_to_str(unsigned char *in)
+char *convert_string(char *str, const char *destset, const char *srcset)
 {
-	int n = 0, i = 0;
-	int inlen;
-	unsigned char *result;
+#ifdef HAVE_ICONV
+	char *buf;
+	iconv_t cd;
+	size_t insize = 0;
+	size_t outsize = 0;
+	size_t nconv = 0;
+	char *inptr;
+	char *outptr;
+	char *ret;
 
-	if (!in)
-		return NULL;
+	if (!str) 
+		return NULL; 
+	buf = g_malloc(strlen(str)*4);
+	insize = strlen(str);
+	inptr = str;
+	outsize = strlen(str)*4;
+	outptr = buf;
 
-	inlen = strlen(in);
-
-	result = g_malloc(inlen + 1);
-
-	while (n <= inlen - 1) {
-		long c = (long)in[n];
-		if (c < 0x80)
-			result[i++] = (char)c;
-		else {
-			if ((c & 0xC0) == 0xC0)
-				result[i++] =
-				    (char)(((c & 0x03) << 6) | (((unsigned char)in[++n]) & 0x3F));
-			else if ((c & 0xE0) == 0xE0) {
-				if (n + 2 <= inlen) {
-					result[i] =
-					    (char)(((c & 0xF) << 4) | (((unsigned char)in[++n]) & 0x3F));
-					result[i] =
-					    (char)(((unsigned char)result[i]) |
-						   (((unsigned char)in[++n]) & 0x3F));
-					i++;
-				} else
-					n += 2;
-			} else if ((c & 0xF0) == 0xF0)
-				n += 3;
-			else if ((c & 0xF8) == 0xF8)
-				n += 4;
-			else if ((c & 0xFC) == 0xFC)
-				n += 5;
-		}
-		n++;
+	cd = iconv_open(destset, srcset);
+	if (cd == (iconv_t) -1) {
+		g_free(buf);
+		debug_printf("iconv_open(%s, %s) Error\n",destset, srcset);
+		return g_strdup(str);
 	}
-	result[i] = '\0';
 
-	return result;
-}
-
-char *str_to_utf8(unsigned char *in)
-{
-	int n = 0, i = 0;
-	int inlen;
-	char *result = NULL;
-
-	if (!in)
-		return NULL;
-
-	inlen = strlen(in);
-
-	result = g_malloc(inlen * 2 + 1);
-
-	while (n < inlen) {
-		long c = (long)in[n];
-		if (c == 27) {
-			n += 2;
-			if (in[n] == 'x')
-				n++;
-			if (in[n] == '3')
-				n++;
-			n += 2;
-			continue;
-		}
-		/* why are we removing newlines and carriage returns?
-		if ((c == 0x0D) || (c == 0x0A)) {
-			n++;
-			continue;
-		}
-		*/
-		if (c < 128)
-			result[i++] = (char)c;
-		else {
-			result[i++] = (char)((c >> 6) | 192);
-			result[i++] = (char)((c & 63) | 128);
-		}
-		n++;
+	nconv = iconv(cd, &inptr, &insize, &outptr, &outsize);
+	if (nconv == (size_t) -1) {
+		debug_printf("iconv Error\n");
+		g_free(buf);
+		return g_strdup(str);
 	}
-	result[i] = '\0';
+	*outptr = '\0';
+	iconv_close(cd);
 
-	return result;
+	ret = g_strdup(buf);
+	g_free(buf);
+
+	return ret;
+#else
+	return g_strdup(str);
+#endif
 }
 
 void strip_linefeed(gchar *text)
